from audiostack.content.script import Script
from audiostack.content.media import Media


def list_projects():
    from audiostack.content.root_functions import Root
    Root.list_projects()
    
def list_modules():
    from audiostack.content.root_functions import Root
    Root.list_projects()


import json

class Response():
    
    statusCode: int
    
    # mututally exclusive
    errors: list = []
    data: dict = {}
    
    meta: dict
    message: str
    warnings: list
    
    def __repr__(self):
        if self.statusCode >= 200:
            return json.dumps({"data" : self.data})
    def __str__(self):
        return "member of Test"



from audiostack.helpers.request_interface import RequestInterface
from audiostack.helpers.request_types import RequestTypes
from audiostack.helpers.api_item import APIResponseItem
from audiostack.helpers.api_list import APIResponseList

from typing import Union

class Sound():
    interface = RequestInterface(family="production/sound")

    # ----------------------------------------- TEMPLATE -----------------------------------------
    class Template():
        
        class Item(APIResponseItem):
            
            def __init__(self, response) -> None:
                super().__init__(response)

                if "template" in self.data: #
                    self.data = self.data["template"]

                self.templateName = self.data["templateName"]
                self.collections = self.data["collections"]
                self.genre = self.data["genre"]
                self.description = self.data["description"]
                self.tempo = self.data["tempo"]
                self.tags = self.data["tags"]
                

        class List(APIResponseList):
            def __init__(self, response, list_type) -> None:
                super().__init__(response, list_type)

            def resolve_item(self, list_type, item):
                if list_type == "templates":
                    return Sound.Template.Item({"data" : item})
                else:
                    raise Exception()

        @staticmethod
        def list(
            tags: Union[str, list] = "", 
            contents: Union[str, list] = "",
            collections: Union[str, list] = "",
            genre: str = "",
            tempo: str = "",
            type: str = "all"
            ) -> list:
            
            if type not in ["all", "custom", "standard"]:
                raise Exception("Invalid type supplied, should be 'all', 'custom', 'standard'")
            
            query_params = {
                "tags" : tags,
                "contents" : contents,
                "collections" : collections,
                "genre" : genre,
                "tempo" : tempo,
                "type" : type
            }
            r = Sound.interface.send_request(rtype=RequestTypes.GET, route="template", query_parameters=query_params)
            return Sound.Template.List(r, list_type="templates")
        

        def create(templateName: str, description: str=""):
            body = {
                "templateName" : templateName,
                "description" : description
            }
            r = Sound.interface.send_request(rtype=RequestTypes.POST, route="template", json=body)
            print(r)
            return Sound.Template.Item(r)


        def delete(templateName: str):
            r = Sound.interface.send_request(rtype=RequestTypes.DELETE, route="template", path_parameters=templateName)
            return APIResponseItem(r)


        def update(
            templateName: str, 
            description: str ="", 
            genre: str="", 
            collections: list=None, 
            tags: list=None):

            body = {
                "templateName" : templateName,
                "description" : description,
                "genre" : genre,
                "collections" : collections,
                "tags" : tags,
            }
            print(body)
            r = Sound.interface.send_request(rtype=RequestTypes.PUT, route="template", json=body)
            return Sound.Template.Item(r)
    
    # ----------------------------------------- TEMPLATE SEGMENT -----------------------------------------
    class Segment():
        def create(mediaId: str, templateName: str, soundSegmentName: str):
            segment = {
                "templateName" : templateName,
                "segmentName" : soundSegmentName,
                "mediaId" : mediaId
            }
            r = Sound.interface.send_request(rtype=RequestTypes.POST, route="segment", json=segment)
            return Sound.Template.Item(r)



    # ----------------------------------------- TEMPLATE PARAMETERS -----------------------------------------
    class Parameter():
        
        @staticmethod
        def get() -> dict:
            r = Sound.interface.send_request(rtype=RequestTypes.GET, route="parameter")
            return APIResponseItem(r)



from audiostack.helpers.request_interface import RequestInterface
from audiostack.helpers.request_types import RequestTypes
from audiostack.helpers.api_item import APIResponseItem

class Root: 
    interface = RequestInterface(family="content")

    def list_projects() -> list:
        r = Root.interface.send_request(rtype=RequestTypes.GET, route="list_projects")
        return APIResponseItem(r)

    def list_modules(projectName: str) -> list:
        r = Root.interface.send_request(rtype=RequestTypes.GET, route="list_projects", query_parameters={"projectName" : projectName})
        return APIResponseItem(r)


from audiostack.helpers.request_interface import RequestInterface
from audiostack.helpers.request_types import RequestTypes
from audiostack.helpers.api_item import APIResponseItem



class Encoder():
    interface = RequestInterface(family="delivery")
    
    class Item(APIResponseItem):
        
        def __init__(self, response) -> None:
            super().__init__(response)
            self.url = self.data["url"]
            self.format = self.data["format"]

        def download(self, fileName="default", path="./") -> None:
            
            full_name = f"{fileName}.{self.format}"
            RequestInterface.download_url(self.url, destination=path, name=full_name)
            
        
    @staticmethod
    def encode_mix(productionId: str="", productionItem: object=None, preset: str="", public: bool=False) -> Item:
                
        if productionId and productionItem:
            raise Exception("productionId or productionItem should be supplied not both")
        if not (productionId or productionItem):
            raise Exception("productionId or productionItem should be supplied")
            
        if productionItem:
            try:
                productionId = productionItem.productionId
            except Exception:
                raise Exception("supplied productionItem is missing an attribute, productionItem should be type object and a response from Production.Mix")
        elif productionId:
            if not isinstance(productionId, str):
                raise Exception("supplied productionId should be a uuid string.")

        body = {
            "productionId": productionId,
            "preset": preset,
            "public": public,
        }
        
        r = Encoder.interface.send_request(rtype=RequestTypes.POST, route="encoder", json=body)
        return Encoder.Item(r)



import json
import shutil

import requests

from audiostack.helpers.util import bcolors
from audiostack.helpers.response import Response
from audiostack.helpers.request_types import RequestTypes

import audiostack


def remove_empty(data):

    if not (isinstance(data, dict) or isinstance(data, list)):
        return data

    final_dict = {}
    for key, val in data.items():
        if val:
            if isinstance(val, dict):
                final_dict[key] = remove_empty(val)
            elif isinstance(val, list):
                final_dict[key] = list(filter(None, [remove_empty(i) for i in val]))
            else:
                final_dict[key] = val
    return final_dict

class RequestInterface:
    
    # disable debug print
    DEBUG_PRINT = False
    

    def __init__(self, family: str) -> None:
        self.family = family

    
    def make_header(self):
        return {
            "x-api-key": audiostack.api_key,
            "x-python-sdk-version": audiostack.sdk_version
            #"x-assume-org": audiostack.assume_org_id,
        }
    

    def resolve_response(self, r):
        if r.status_code >= 500:
            raise Exception("Internal server error - aborting")

        if self.DEBUG_PRINT:
            print(json.dumps(r.json(), indent=4))
        
        if "meta" in r.json():
            if "creditsUsed" in r.json()["meta"]:
                audiostack.billing_session += r.json()["meta"]["creditsUsed"]

        if r.status_code == 403:
            raise Exception("Not authorised - check API key is valid")
        
        if r.status_code >= 400:
            msg = r.json()["message"] + ". Errors listed as follows: \n\t" + '\t'.join(r.json()["errors"])
            raise Exception(msg)

        return {**r.json(), **{"statusCode" : r.status_code}}

    
    def send_upload_request(self, local_path, upload_url):
        with open(local_path, 'rb') as data:
            r = requests.put(url=upload_url, data=data)

            if r.status_code >= 400:
                raise Exception("Failed to upload file")

            return r.status_code


    def send_request(self, rtype, route, json=None, path_parameters=None, query_parameters=None, overwrite_base_url=None):
        
        if overwrite_base_url:
            url = overwrite_base_url
        else:
            if self.family:
                url =  f"{audiostack.api_base}/{self.family}"
            else:
                url = audiostack.api_base
        
        if route:
               url += "/" + route 

        if rtype not in RequestTypes.valid_types:
            assert False
        
        if json:
            json = remove_empty(json)
        if query_parameters:
            query_parameters = remove_empty(query_parameters)
        
        # these requests are all the same input parameters.
        if rtype in [RequestTypes.POST, RequestTypes.PUT, RequestTypes.PATCH]:
            
            FUNC_MAP = {
                RequestTypes.POST : requests.post,
                RequestTypes.PUT : requests.put,
                RequestTypes.PATCH : requests.patch
            }
            
            return self.resolve_response(
                FUNC_MAP[rtype](
                    url=url, 
                    json=json, 
                    headers=self.make_header()
                )
            )
        elif rtype == RequestTypes.GET:
            
            if path_parameters:
                url = f"{url}/{path_parameters}"
                
            return self.resolve_response(
                requests.get(
                    url=url,
                    params=query_parameters,
                    headers=self.make_header()
                )
            )
        elif rtype == RequestTypes.DELETE:
            
            if path_parameters:
                url = f"{url}/{path_parameters}"

            return self.resolve_response(
                requests.delete(
                    url=url,
                    params=query_parameters,
                    headers=self.make_header()
                )
            )
    
    @staticmethod
    def download_url(url, name, destination):
        r = requests.get(
            url=url, 
            stream=True, 
            headers={"x-api-key": audiostack.api_key}
        )

        if r.status_code >= 400:
            raise Exception("Failed to download file")

        local_filename = f"{destination}/{name}"
        with open(local_filename, "wb") as f:
            shutil.copyfileobj(r.raw, f)


from audiostack.helpers.request_interface import RequestInterface
from audiostack.helpers.request_types import RequestTypes
from audiostack.helpers.api_item import APIResponseItem
from audiostack.helpers.api_list import APIResponseList

class Documentation():
    
    interface = RequestInterface(family="")
    
    @staticmethod
    def docs_for_service(service: object) -> dict:
        service = service.__name__.lower()

        r = Documentation.interface.send_request(rtype=RequestTypes.GET, route="documentation", query_parameters={"route" : service})
        return APIResponseItem(r)
        




from audiostack.helpers.request_interface import RequestInterface
from audiostack.helpers.request_types import RequestTypes
from audiostack.helpers.api_item import APIResponseItem
from audiostack.helpers.api_list import APIResponseList

class TTS():

    interface = RequestInterface(family="speech")
    
    class Item(APIResponseItem):
        
        def __init__(self, response) -> None:
            super().__init__(response)
            self.speechId = self.data["speechId"]

        def download(self, autoName=False, fileName="", path="./") -> None:
            
            sections = self.data["sections"]
            for i, s in enumerate(sections):
                if autoName:
                    full_name = ""
                    for k, val in s["audience"].items():
                        full_name += f"{k}={val}~"
                    
                    full_name = full_name[:-1] + ".wav"
                else:
                    if not fileName:
                        full_name = s["sectionName"] + ".wav"
                    else:
                        full_name = f"{fileName}_{i+1}_of_{len(sections)}.wav"
                RequestInterface.download_url(s["url"], destination=path, name=full_name)
                
        def delete(self):
            return TTS.delete(self.speechId)
        
    class List(APIResponseList):
        def __init__(self, response, list_type) -> None:
            super().__init__(response, list_type)

        def resolve_item(self, list_type, item):
            if list_type == "speechIds":
                return TTS.Item({"data" : item})
            else:
                raise Exception()

    
    @staticmethod
    def create(
        scriptId="", 
        scriptItem=None, 
        voice: str="", 
        speed: float=1.0,
        silencePadding: str="", 
        effect: str="", 
        audience: dict={},
        sections: dict={},
        useDictionary: bool=False,
        useTextNormalizer: bool=False,
        public: bool=False,
        sync: bool=True
        ) -> Item:
                
        if scriptId and scriptItem:
            raise Exception("scriptId or scriptItem should be supplied not both")
        if not (scriptId or scriptItem):
            raise Exception("scriptId or scriptItem should be supplied")
            
        if scriptItem:
            scriptId = scriptItem.scriptId
        
        if not isinstance(voice, str):
            raise Exception("voice argument should be a string")        
        if not isinstance(effect, str):
            raise Exception("effect argument should be a string")
        if not isinstance(silencePadding, str):
            raise Exception("silencePadding argument should be a string")
        if not isinstance(useDictionary, bool):
            raise Exception("useDictionary argument should be a boolean")
        if not isinstance(useTextNormalizer, bool):
            raise Exception("useTextNormaliser argument should be a boolean")

        body = {
            "scriptId": scriptId,
            "voice" : voice,
            "speed" : speed,
            "silencePadding" : silencePadding,
            "effect" : effect,
            "audience" : audience,
            "sections" : sections, 
            "useDictionary" : useDictionary,
            "useTextNormalizer" : useTextNormalizer,
            "public" : public,
            "sync" : sync
        }
        
        r = TTS.interface.send_request(rtype=RequestTypes.POST, route="tts", json=body)
        return TTS.Item(r)


    @staticmethod
    def get(speechId: str) -> Item:
        
        r = TTS.interface.send_request(rtype=RequestTypes.GET, route="tts", path_parameters=speechId)
        return TTS.Item(r)


    @staticmethod
    def delete(speechId: str) -> str:
        r = TTS.interface.send_request(rtype=RequestTypes.DELETE, route="tts", path_parameters=speechId)
        return APIResponseItem(r)
    
    
    @staticmethod
    def list(projectName="", moduleName: str="", scriptName: str="", scriptId: str="") -> list:
        query_params = {
            "projectName" : projectName,
            "moduleName" : moduleName,
            "scriptName" : scriptName,
            "scriptId" : scriptId
        }
        r = TTS.interface.send_request(rtype=RequestTypes.GET, route="tts", query_parameters=query_params)
        return TTS.List(r, list_type="speechIds")


from audiostack.helpers.request_interface import RequestInterface
from audiostack.helpers.request_types import RequestTypes
from audiostack.helpers.api_item import APIResponseItem
from audiostack.helpers.api_list import APIResponseList

class Mix():

    interface = RequestInterface(family="production")
    
    class Item(APIResponseItem):
        
        def __init__(self, response) -> None:
            super().__init__(response)
            self.productionId = self.data["productionId"]

        def download(self, fileName="", path="./") -> None:
            
            sections = self.data["files"]
            for i, s in enumerate(sections):
                format = s["format"]
                if not fileName:
                    full_name = f"default_mix.{format}"
                else:
                    full_name = f"{fileName}.{format}"

                RequestInterface.download_url(s["url"], destination=path, name=full_name)
                
        def delete(self):
            return Mix.delete(self.productionId)
 

    class List(APIResponseList):
        def __init__(self, response, list_type) -> None:
            super().__init__(response, list_type)

        def resolve_item(self, list_type, item):
            if list_type == "productionIds":
                return Mix.Item({"data" : item})
            else:
                raise Exception()

    @staticmethod
    def create(
        speechId="", 
        speechItem=None, 
        soundTemplate: str="",
        forceLength: float=0.0, # a value of 0 means no force length
        mediaFiles: dict={},
        sectionProperties: dict={},
        masteringPreset: str="",
        acousticSpace: str="",
        callbackUrl: str="",
        public: bool=False,
        ) -> Item:
                
        if speechId and speechItem:
            raise Exception("speechId or scriptItem should be supplied not both")
        if not (speechId or speechItem):
            raise Exception("speechId or scriptItem should be supplied")
        if speechItem:
            speechId = speechItem.speechId

        if not isinstance(soundTemplate, str):
            raise Exception("soundTemplate argument should be a string")
        if not isinstance(forceLength, float):
            raise Exception("forceLength should be a float")
        if not isinstance(masteringPreset, str):
            raise Exception("masteringPreset should be a string")
        if not isinstance(acousticSpace, str):
            raise Exception("acousticSpace should be a string")
        
        
        body = {
            "speechId": speechId,
            "soundTemplate" : soundTemplate,
            "forceLength": forceLength, 
            "mediaFiles": mediaFiles, 
            "sectionProperties": sectionProperties,
            "masteringPreset": masteringPreset,
            "acousticSpace": acousticSpace,
            "callbackUrl": callbackUrl,
            "public": public
        }
        
        r = Mix.interface.send_request(rtype=RequestTypes.POST, route="mix", json=body)
        return Mix.Item(r)


    @staticmethod
    def get(productionId: str) -> Item:
        
        r = Mix.interface.send_request(rtype=RequestTypes.GET, route="mix", path_parameters=productionId)
        return Mix.Item(r)


    @staticmethod
    def delete(productionId: str) -> str:
        r = Mix.interface.send_request(rtype=RequestTypes.DELETE, route="mix", path_parameters=productionId)
        return APIResponseItem(r)


    @staticmethod
    def list(projectName="", moduleName: str="", scriptName: str="", scriptId: str="") -> list:

        query_params = {
            "projectName" : projectName,
            "moduleName" : moduleName,
            "scriptName" : scriptName,
            "scriptId" : scriptId
        }

        r = Mix.interface.send_request(rtype=RequestTypes.GET, route="mixes", query_parameters=query_params)
        return Mix.List(r, list_type="productionIds")
    


import json

class APIResponseItem():
    
    def __init__(self, response):
        self.response = response
        
        if "data" in response:
            self.data = self.response["data"]
        if "message" in response:
            self.message = self.response["message"]
        if "meta" in response:
            self.meta = self.response["meta"]

            # for key in self.data:
            #     assert key != "data"
            #     self.__dict__[key] = self.data["key"]
    
    def print_response(self, indent=0):
        if indent:
            return json.dumps(self.response, indent=indent)
        else:
            return self.response
    
    def __str__(self) -> str:
        return json.dumps(self.response)


from audiostack.helpers.request_interface import RequestInterface
from audiostack.helpers.request_types import RequestTypes
from audiostack.helpers.api_item import APIResponseItem
from audiostack.helpers.api_list import APIResponseList

class Script():
    FAMILY = "content"
    interface = RequestInterface(family=FAMILY)
        
    class Item(APIResponseItem):
        
        def __init__(self, response) -> None:
            super().__init__(response)
            
            self.scriptId = self.data["scriptId"]
            self.projectName = self.data["projectName"]
            self.moduletName = self.data["moduleName"]
            self.scriptName = self.data["scriptName"]
            self.scriptText = self.data["scriptText"]
                
        def update(self, scriptText):
            return Script.update(scriptId=self.scriptId, scriptText=scriptText)

        def delete(self):
            return Script.delete(self.scriptId)
        

    class List(APIResponseList):
        
        def __init__(self, response, list_type) -> None:
            super().__init__(response, list_type)

        def resolve_item(self, list_type, item):
            if list_type == "scripts":
                return Script.Item({"data" : item})
            else:
                raise Exception()


    @staticmethod
    def create(scriptText: str, projectName: str="", moduleName: str="", scriptName: str="", metadata: dict={}) -> Item:
        
        body = {
            "scriptText" : scriptText,
            "projectName" : projectName,
            "moduleName" : moduleName,
            "scriptName" : scriptName,
            "metadata" : metadata
        }
        
        r = Script.interface.send_request(rtype=RequestTypes.POST, route="script", json=body)
        return Script.Item(r)
    

    @staticmethod
    def get(scriptId: str, version: str="", previewWithVoice: str="") -> Item:
        
        path_params = f"{scriptId}/{version}" if version else scriptId
        query_params = {"preview" : bool(previewWithVoice), "voice" : previewWithVoice}
        r = Script.interface.send_request(
            rtype=RequestTypes.GET, 
            route="script", 
            path_parameters=path_params,
            query_parameters=query_params
        )
        return Script.Item(r)
    

    @staticmethod
    def delete(scriptId: str, version: str="") -> str:
        
        path_params = f"{scriptId}/{version}" if version else scriptId
        r = Script.interface.send_request(rtype=RequestTypes.DELETE, route="script", path_parameters=path_params)
        return APIResponseItem(r)


    @staticmethod
    def update(scriptId: str, scriptText, version: str="") -> Item:
        body = {
            "scriptId" : scriptId,
            "scriptText" : scriptText,
            "version" : version
        }
        r = Script.interface.send_request(rtype=RequestTypes.PUT, json=body, route="script")
        return Script.Item(r)

    @staticmethod
    def update_section(scriptId: str, text: str, sectionName: str):
        pass

    @staticmethod
    def list(projectName="", moduleName: str="", scriptName: str="", scriptId: str="") -> List:
        query_params = {
            "projectName" : projectName,
            "moduleName" : moduleName,
            "scriptName" : scriptName,
            "scriptId" : scriptId
        }
        r = Script.interface.send_request(rtype=RequestTypes.GET, route="scripts", query_parameters=query_params)
        return Script.List(r, list_type="scripts")



from audiostack.helpers.request_interface import RequestInterface
from audiostack.helpers.request_types import RequestTypes
from audiostack.helpers.api_item import APIResponseItem
from audiostack.helpers.api_list import APIResponseList
import os
import requests

class Media():
    FAMILY = "content"
    interface = RequestInterface(family=FAMILY)
        
    class Item(APIResponseItem):
        
        def __init__(self, response) -> None:
            super().__init__(response)
            
            self.mediaId = self.data["mediaId"]
            self.tags = self.data["tags"]
            self.filename = self.data["filename"]
                

        def delete(self):
            return Media.delete(self.mediaId)
        

    class List(APIResponseList):
        
        def __init__(self, response, list_type) -> None:
            super().__init__(response, list_type)

        def resolve_item(self, list_type, item):
            if list_type == "mediaFiles":
                return Media.Item({"data" : item})
            else:
                raise Exception()


    @staticmethod
    def create(filePath: str) -> Item:

        if not os.path.isfile(filePath):
            raise Exception("Supplied file does not eixst")
        
        name = filePath.rpartition("/")[2]
         
        r = Media.interface.send_request(rtype=RequestTypes.GET, route="media/create-upload-url", path_parameters=name)
        response = APIResponseItem(r)
        url = response.data["fileUploadUrl"]
        mediaId = response.data["mediaId"]

        status = Media.interface.send_upload_request(local_path=filePath, upload_url=url)
        return Media.get(mediaId)


    @staticmethod
    def get(mediaId: str) -> Item:

        r = Media.interface.send_request(rtype=RequestTypes.GET, route="media", path_parameters=mediaId)
        return Media.Item(r)
    

    @staticmethod
    def delete(mediaId: str) -> str:

        r = Media.interface.send_request(rtype=RequestTypes.DELETE, route="media", path_parameters=mediaId)
        return APIResponseItem(r)


    @staticmethod
    def list() -> List:

        r = Media.interface.send_request(rtype=RequestTypes.GET, route="media")
        return Media.List(r, list_type="mediaFiles")

